#! /usr/bin/perl
######################################################################
# Copyright 1995, 1996 Ipsilon Networks, Inc. All rights reserved.
# 
# YOU MAY ONLY USE THE MATERIALS FOUND IN THIS FILE PURSUANT TO THE
# TERMS OF THE EVALUATION LICENSE AGREEMENT FOUND IN "/LICENSE".  ALL
# OTHER USES ARE EXPRESSLY PROHIBITED.
######################################################################

# Global associative arrays used (based on interface name):
#	iflcladdr, ifmsklen, iflink, ifremaddr, ifseen
#	ifispnp, ifpnpremote
#       ipxaddr, ipxifseen 
#       aliasifseen

require 'syslog.pl';

#===============================================================
#       
#                 SUBROUTINE hsleep 
#          
#  hsleep allows a sleep call in 1/100 sec intervals
#       
#  RETURN VALUE: void
#
#  call syntax: &hsleep ($time);
#       
#===============================================================    
      
sub hsleep { 
                 
    local ($time) = @_;        
   
    $time *= 1.0;
    if ($time > 0)  {
       select (undef, undef, undef, $time);
    }   
    undef;
}   
#
# Secondary addressing (interface aliases) note:
#         Some of the implementation is redundant in order to
#         keep the functionality isolated instead of interweaving 
#         it with the rest. 
#

# Called when we're going to exit: don't call "die" directly

sub fatal_error {
    local($message) = @_;
    &syslog(LOG_ERR, $message);
    die $message;
}

sub stop_mrouted {
        system("/usr/local/config/bin/udw machine:$hostname:process:mrouted");
}
sub stop_ipsrd {
        system("/usr/local/config/bin/udw machine:$hostname:process:ipsrd");
}
sub start_ipsrd {
        system("/usr/local/config/bin/udw \"machine:$hostname:process:ipsrd t\"");
}
sub start_fw {
	$fwdir = "/etc/fw/";
	return unless -d "$fwdir/bin";
	# Is firewall turned on?
	$fwlic = `/usr/local/config/bin/dbget "machine:$hostname:license:cpfw"`;
	if ($fwlic) {
	    $fwstate = `/usr/local/config/bin/dbget "machine:$hostname:cpfw:state"`;
	    if ($fwstate) {
		&syslog(LOG_ERR, "Starting Check Point FireWall-1\n");

		# read the master file
		if ( -f "$fwdir/conf/masters" ) {
			$masters = `/bin/cat $fwdir/conf/masters`;
		}

		$fwdrc = system("$fwdir/bin/fwd $masters");
		if ($fwdrc) {
			&syslog(LOG_ERR, "FW-1: failed to start fwd\n");
			return 1;
		}

		system ("nohup $fwdir/bin/snmpd > /dev/null");

		$fwdrc = system("$fwdir/bin/fw fetch $masters");
		if ($fwdrc) {
			&syslog(LOG_ERR,"FW-1 fetch from $masters failed, trying local copy\n");
			$fwdrc = system("$fwdir/bin/fw fetch localhost");
		}
		if ($fwdrc) {
			&syslog(LOG_ERR, "\nCheck Point FireWall-1 startup failed. administrator must manually load rule set\n");
			return 1;
		} 
		&syslog(LOG_ERR, "\nCheck Point FireWall-1 is up.\n");
		return;
	    }
	}
	return 1;
}

# Toggle debug
sub sigusr1_handler {	# 1st argument is signal name
	local($sig) = @_;
	$debug= ! $debug;
	&syslog(LOG_NOTICE, "Debug set to $debug\n");
}

# Reconfigure 
sub sighup_handler {	# 1st argument is signal name
	local($sig) = @_;

	if ($scanifs_in_progress == 0) {
                &syslog(LOG_NOTICE, "sighup_handler: call handler") if $debug; 
		&handler(0);
	} else {
		$signal_pending = 1; 
		$interval = 1;
	}
	return;
}

sub handler {
        local ($isinit) = @_; 
        local($isup, $islink1, $islink2);

	&syslog(LOG_NOTICE, "Received reconfiguration request\n");
        print "Received reconfiguration request\n" if $debug; 
	# See the startup to see why some of these things are done	
	&parse_addrconf;
        &parse_ipxaddrconf;
	open(IFCONFIG, "/sbin/ifconfig -a |") || return;
	while (<IFCONFIG>) {
		chop;

		$isup = $islink1= $islink2= 0;

		# Scan through each interface line looking for 
		# interface names we don't know about
		if (/^(.*):.*flags=.*<(.*)>/) {
   			@intfflags = split(/,/, $2);
			foreach $i (0 .. $#ifflags) {
				if ($intfflags[$i] eq "UP") {
					$isup = 1;
				}
				if ($intfflags[$i] eq "LINK1") {
					$islink1 = 1;
				}
				if ($intfflags[$i] eq "LINK2") {
					$islink2 = 1;
				}
			}
			$ipintf  = &is_known_iface($1);
			$ipxintf = &is_known_ipxiface($1);
                        $aliasintf = &is_known_aliasiface($1);
			
			if ( !$isup  && ($1 eq $mcast_if)) {
			        &syslog(LOG_NOTICE, "Deleting multicast route.\n");
				system "/sbin/route delete 224.0.0.0 -netmask 0xf0000000";
				$need_mcast_route = 1;
				$mcast_if = "";
			}
			if (!$ipintf && $ipxintf) {
			        &syslog(LOG_NOTICE, 
					"Deleting $1's IP address\n") if $debug;
			        $ifcmd{$1}= "";
				$ifconf{$1} = "";
				system "/sbin/ifconfig $1 inet delete";
				sleep 1;
			}
			elsif ($ipintf && !$ipxintf) {
			        &syslog(LOG_NOTICE, 
				      "Deleting $1's IPX address\n") if $debug;
			        $ipxcmd{$1}= "";
				$ipxifconf{$1} = "";
				system "/sbin/ifconfig $1 ipx delete";
				$ipxseen{$1} = 0;
				sleep 1;
			}
			elsif (!$ipintf && !$ipxintf && ($islink2 || $islink1) && $isup) {
			       &syslog(LOG_NOTICE, 
				       "Deleting $1's IP & IPX addresses\n") 
				       if $debug;
			       $ipxcmd{$1}= ""; $ifcmd{$1}= "";
			       $ifconf{$1} = ""; $ipxifconf{$1} = "";
			       system "/sbin/ifconfig $1 inet delete";
			       system "/sbin/ifconfig $1 ipx delete";
			}
			elsif (!$ipintf && !$ipxintf && !$islink2 && !$islink1 && $isup) {
			        &syslog(LOG_NOTICE, 
					"Deleting $1's IP & IPX addresses\n")
					if $debug;
			        $ipxcmd{$1}= ""; $ifcmd{$1}= "";
				$ifconf{$1} = ""; $ipxifconf{$1} = "";
			        system "/sbin/ifconfig $1 down";
			        system "/sbin/ifconfig $1 inet delete";
			        system "/sbin/ifconfig $1 ipx delete";
				if ($1 =~ /^atm/) {
					system "/sbin/ifconfig $1 delif";
				}
			        sleep 1;
			}
		}
	} 
	close(IFCONFIG);
	# Delay alias commands until now
        # Clear alias command and ipaddr to rebuild secondary lists. 
	if(!$isinit) {
	    &syslog(LOG_NOTICE, "handler: up (not init)"); 
	    undef %aliascmd;
	    undef %aliasaddr; 	         
            &parse_aliasaddrconf; 
        }
        else {
            &syslog(LOG_NOTICE, "handler: NOT up (init)");  
	}
	$interval = 10;
}

sub is_known_iface {
	local($iface, %localaddr) = @_;

	# Return true if the given interface is in the config file last
	# parsed or is special cased.  At this point the only special
	# case interface is the loopback interface lo0.

	# Check for special cases first
	if ($iface eq "lo0") {
		return 1;
	}
	
	return $ifseen{$iface};
}

sub is_known_ipxiface {
	local($iface) = @_;

	# Return true if the given interface is in the config file last
	# parsed or is special cased.  At this point the only special
	# case interface is the loopback interface lo0.

	# Check for special cases first
	if ($iface eq "lo0") {
		return 1;
	}
	
	return $ipxifseen{$iface};
}

sub is_known_aliasiface {
	local($iface, %localaddr) = @_;
	# Return true if the given interface is in the config file last
	# parsed or is special cased.  At this point the only special
	# case interface is the loopback interface lo0.

	# Check for special cases first
	if ($iface eq "lo0") {
		return 1;
	}	
	return $aliasifseen{$iface};
}

sub get_local {
        local($iface) = @_;

        open(IFACE, "/sbin/ifconfig $iface |") || return ;
        &syslog(LOG_NOTICE, "get_local $iface") if $debug;
        while (<IFACE>) {
                chop;
                # Look for IP address of remote side (take first address)
                if (/^\s*inet\s+(\S+)\s+/) {
                        &syslog(LOG_NOTICE, "get_local $1") if $debug;
                        return $1;
                }
        }
        close(IFACE);
        return "0.0.0.0";
}

sub get_remote {
	local($iface) = @_;

	open(IFACE, "/sbin/ifconfig $iface |") || return;
	while (<IFACE>) {
		chop;
		# Look for IP address of remote side (take first address)
		if (/^\s*inet\s+(\S+)\s+-->\s+(\S+)/) {
			return $2;
		}
	}
	close(IFACE);
	return "0.0.0.0";
}

sub start_mrouted {

	if ($need_mcast_route) {
		&syslog(LOG_NOTICE, "Need multicast route to start mrouted\n");
		return;
	}

	return unless -e "/etc/mrouted.conf";

	&stop_mrouted;
	# start it up again
        system("/usr/local/config/bin/udw \"machine:$hostname:process:mrouted t\"");
};

sub configaliasif {
    local($iface, $prompt, $addr) = @_;

    &syslog(LOG_NOTICE, "Configuring ALIAS address for $iface\n") if $debug;
    if($ifalias{$1} eq $iface) {
	$cmd  = "/sbin/ifconfig $iface $prompt $addr";      
        &syslog(LOG_NOTICE, "alias config command = $cmd\n") if $debug; 
	if ($aliascmd{$iface} ne $cmd) {
	    $aliascmd{$iface} = $cmd;
            &syslog(LOG_NOTICE, "exec'ing $cmd \n") if $debug;
	    system $cmd;
	    &hsleep (0.2);  # sleep for 1/5 sec
            # sleep 1; 
	}
    }
}

sub configipxif {
    ($iface) = @_;

    &syslog(LOG_NOTICE, "IPX address for $iface is $ipxaddr{$iface}\n") if $debug;

    if ($ipxaddr{$iface}) {
	$cmd = "/sbin/ifconfig $iface ipx $ipxaddr{$1}";
	if ($ipxlname{$iface}) {
	    $cmd.=" lname $ipxlname{$iface}";
	}
	$cmd .= " up";
	&syslog(LOG_NOTICE, "command '$cmd'\n") if $debug;
	if ($ipxcmd{$iface} ne $cmd) {
	    $ipxcmd{$iface}= $cmd;
	    system $cmd;
	    sleep 1;
	}
    }
}

sub configif {
	($iface) = @_;
	&syslog(LOG_NOTICE, "Configuring IP address for $iface\n") if $debug;
        print "Configuring IP address for $iface\n" if $debug;

	if ($iflcladdr{$iface}) {
	    $change_ifconf = 1;
	    $cmd = "/sbin/ifconfig $iface $iflcladdr{$1}";
	    if ($ifmsklen{$iface}) {
		$cmd .= "/$ifmsklen{$iface}";
	    }
	    &syslog(LOG_NOTICE, "$iface is a P2P interface\n") if $debug;
	    if ($ifremaddr{$iface}) {
		    $cmd .= " $ifremaddr{$iface}";
	    }
	    elsif ($iface =~ /^atm/) {
		&syslog(LOG_NOTICE, "$iface PNPIF\n") if $debug;
		$cmd .= " pnpif";
		$ifispnp{$iface} = 1;
		$ifpnpremote{$iface} = "0.0.0.0";
	    }
	    if ($iflname{$iface}) {
		$cmd.=" lname $iflname{$iface}";
	    }
	    if ($ififmp{$iface}) {
		$cmd.=" $ififmp{$iface}";
	    } elsif ($iface =~ /^atm/) {
		$cmd.=" ifmp";
	    }
	    $cmd .= " $ifduplex{$iface} $iflink{$iface} up";
	    &syslog(LOG_NOTICE, "command '$cmd'\n") if $debug;
	    if ($ifcmd{$iface} ne $cmd) {
		$st = system $cmd;
		$ifcmd{$iface}= $cmd;
		sleep 1;
	    }
	} else {
	    $cmd = "/sbin/ifconfig $iface ";
	    if ($iflname{$iface}) {
		$cmd.=" lname $iflname{$iface}";
	    }
	    $cmd .= " $ifduplex{$iface} $iflink{$iface} up";
	    &syslog(LOG_NOTICE, "command '$cmd'\n") if $debug;
	    $st = system $cmd;
	    $ifcmd{$iface}= $cmd;
	    sleep 1;
	}
}

# IFMP packets are only sent if the interface is marked up.
# The goal here is to configure an interface we have configuration
# information on when we first encounter the interface.  We must
# also mark the interface up as we configure the address.
# With IFMP it is assumed the RUNNING flag is an indication
# of the IFMP adjacency being established.
# Interfaces that we have configuration information on are
# only ever configured and marked up the first time they
# are reported.
#
# For P2P Ipsilon interfaces (defined by interface name starting with 'ips')
# we configure only the local side and mark the interface up.
# This causes the IFMP adjacency process to start.
# Note in this case no interface route is installed, we rely on
# 'ipsrd' to pick up the interface when fully configured and install
# an appropriate route.

$ADD_224_0_0_0 = "/sbin/route add 224.0.0.0 -netmask 0xf0000000 -interface ";

sub scanifs {
	local($isptp,$ismcast,$pnpremote,$isloop,$isup);

	$SIG{'HUP'} = 'sighup_handler';
	open(IFCONFIG, "/sbin/ifconfig -a |") || return;

	&parse_addrconf;
	&parse_ipxaddrconf;        
	$scanifs_in_progress = 1;
	$new_if = 0;

	@iface_keys = keys %ifseen;
	foreach $key (@iface_keys) {
		if ((!$ifactive{$key}) && $ifcreate{$key}) {
		        &syslog(LOG_NOTICE, "scanifs: running addif on $key\n")
			    if $debug;
			system("/sbin/ifconfig $key addif");
		}
	}

	while (<IFCONFIG>) {
		chop;
		# Scan through each interface line looking for interface
		# names and flags
		if (/^(.*):.*flags=.*<(.*)>/) {
			$isptp = 0;
			$ismcast = 0;
			$isloop = 0;
			$isup = 0;

			@ifflags = split(/,/, $2);
			foreach $i (0 .. $#ifflags) {
				if ($ifflags[$i] eq "POINTOPOINT") {
					$isptp = 1;
				}
				if ($ifflags[$i] eq "MULTICAST") {
					$ismcast = 1;
				}
				if ($ifflags[$i] eq "LOOPBACK") {
					$isloop = 1;
				}
				if ($ifflags[$i] eq "UP") {
					$isup = 1;
				}
			}
			if ($ifseen{$1} && !$ifactive{$1}) {
				# We have not seen this interface

			        # try to configure it and mark up
				&configif($1);
				$new_if = 1;
			}
			if ($ifactive{$1} && $ifseen{$1}) {
				# we have seen this one since last config.
				# inform mrouted if change of remote address
				if ($ifispnp{$1}) {
					$pnpremote = &get_remote($1);
					if ($pnpremote ne 
					    	$ifpnpremote{$1}) {
						# remote side of pnpif 
						# has changed
						$ifpnpremote{$1} = $pnpremote;
						$change_ifconf = 1;
					}
				}
			}
			# install 224 route on first mcast interface discovered
	   		if ($isup && $iflcladdr{$1} && 
			    $ismcast && $need_mcast_route && !$isloop) {
				$need_mcast_route = 0;
				$mcast_if = $1;	# optimize deletion of 224 route
				system "$ADD_224_0_0_0 $iflcladdr{$1}";
	    		}
			if ($ipxifseen{$1} && !$ifactive{$1}) {
				# We have not seen this interface
				# since last update of the ipxif.conf,
			        # try to configure it and mark up
				&configipxif($1);
				$new_if = 1;
			}                        
			$ifactive{$1} = 1;
		}
	}
	if ($new_if == 1) {
            system("/usr/local/config/bin/policy /etc/policy.conf");
            system("/usr/local/config/bin/policy /etc/rs.conf");
	}
	close(IFCONFIG);
        # Delay alias commands until now. 	
        &parse_aliasaddrconf;	
	$scanifs_in_progress = 0;
}

# ifconfig <if> <addr>[/<masklen>] -?link1 -?link2 [-p2p <addr>] [lname foo] [-ifmp] [addif]

$key = '^\s*ifconfig\s+';
$ipa = '\d+\.\d+\.\d+\.\d+';
$ifn = '[\w-]+';
$opt = '\s+-?link2';
$dup = '\s+-?link1';
$ifmp = '\s+-?ifmp';

sub parse_addrconf {
	local($linenum,$err,$cmd,$new_config);
	# Clear out any old state we might have
	undef %ifseen;		# interface in if.conf
#	undef %ifispnp;		# pnpif option in if.conf
#	undef %iflcladdr;	# local address in if.conf
#	undef %ifmsklen;	# mask len in if.conf
#	undef %ifduplex;	# duplex option in if.conf
#	undef %iflink;		# link option in if.conf
#	undef %iflname;		# logical name in if.conf
#	undef %ifremaddr;	# remote addr in if.conf
#	undef %ifcreate;	# addif option in if.conf
#	undef %ifpnpremote;	# pnp remote address in live system
#	undef %ifactive;	# interface in live system

	open(ADDRCONF, $path_addrconf) || &fatal_error("Can't open $path_addrconf\n");
	&syslog(LOG_NOTICE, "Parsing '$path_addrconf'\n") if $debug;
	$linenum = 0;
	while (<ADDRCONF>) {
		chop;
		$linenum++;
		if (/^#/ || /^s*$/) {
			next;
		}
                print "$linenum: $_ \n" if $debug;
                # Skip interface aliases for special handling
                if(/alias/) {
		    print "$linenum: $_ \n" if $debug;
                    next; 
                }
		if (!/$key($ifn)(\s+$ipa)?((\/)(\d+))?($dup)?($opt)(\s+-p2p\s+($ipa))?(\slname(\s\S*))?($ifmp)?(\saddif)?/) {
		    print stderr 
			"Error parsing $path_addrconf on line #$linenum: '$_'\n" if $debug;
		    &syslog(LOG_ERR, 
			    "Error parsing $path_addrconf on line #$linenum: '$_'\n");
				$err++;
		    next;
		}		
		if ($13 && ($ifactive{$1} ne 1)) {
			if (system("/sbin/ifconfig $1 addif") != 0) {
				next;
			}
		}
		if ($2) {
		        $ifseen{$1} = 1;
		} else {
			$ifseen{$1} = 0;
		}
		if ($_ eq $ifconf{$1})  {
			next;
		}               

		$iflcladdr{$1} = $2;
		$ifmsklen{$1} = $5;
		$ifremaddr{$1} = $9;
		$ifduplex{$1} = $6;
		$iflink{$1} = $7;
		$iflname{$1} = $11;
		$ifcreate{$1} = $13;

		if ($12) {
			$ififmp{$1} = $12;
	        } elsif ($1 =~ /^atm/) {
			$ififmp{$1} = " ifmp";
		}
		if ($ifactive{$1}) {
			&configif($1);
		}
		$ifconf{$1} = $_;
		if ($debug) { # XXX this should use syslog                 
			print "if: $1 addr: $iflcladdr{$1}";
			if ($ifmsklen{$1}) {
				printf "/$ifmsklen{$1}";
			}
			if ($ifremaddr{$1}) {
				print " -> $ifremaddr{$1}";
			}
			print "\n";
		}
	}
	close(ADDRCONF);

	if ($err) {
		&fatal_error("$err error(s) encountered in parsing $path_addrconf\n");
	}
}

# ifconfig <if> ipx <netaddr>#<hostaddr> [lname foo] [-ifmp] [addif]
$ipxkey = '^\s*ifconfig\s+';
$ipxn = 'ipx';
$ipxnet = '[0-9|a-f|A-F|:|#|x]+';

sub parse_ipxaddrconf {
	local($linenum,$err);
	# Clear out any old state we might have

#	undef %ipxaddr;
#	undef %ipxlname;
	undef %ipxifseen;

	open(IPXADDRCONF, $path_ipxaddrconf) || &fatal_error("Can't open $path_ipxaddrconf\n");
	&syslog(LOG_NOTICE, "Parsing '$path_ipxaddrconf'\n") if $debug;
	$linenum = 0;
	while (<IPXADDRCONF>) {
		chop;
		$linenum++;
		if (/^#/ || /^s*$/) {
			next;
		}
		if (!/$ipxkey($ifn)\s+($ipxn)\s+($ipxnet)?(\slname(.*))?/) {
			&syslog(LOG_ERR, 
				"Error parsing $path_addrconf on line #$linenum: '$_'\n");
			$err++;
			next;
		}
		# mac address should be extracted from the live database
		# only in the case of ethernet or fddi interfaces. Immediately
		# after reboot fefm interfaces may not be created to extract
		# the mac address from the ethernet ports on the EAR cards &
		# we need to skip configuring those ethernet interfaces
		$linkaddr = 0;
		$ifprefix = substr($1, 0, 4);
		if (($ifprefix eq "eth-") || ($ifprefix eq "fddi")) {
		    $linkaddr = `/usr/local/config/bin/dbget "machine:$hostname:ipxinterface:$1:linkaddr"`;
		    if ($linkaddr eq 0) {
			next;
		    }
		}
		$ipxifseen{$1} = 1;
		if ($_ eq $ipxifconf{$1}) {
			next;
		}
		$ipxifconf{$1} = $_;
		$ipxaddr{$1} = $3."#".$linkaddr;
		$ipxlname{$1} = $5;

		if ($ifactive{$1}) {
			&configipxif($1);
		}
		if ($debug) { # XXX this should use syslog
			print "if: $1 addr: $ipxaddr{$1}";
			print "\n";
		}
	}
	close(IPXADDRCONF);

	if ($err) {
		&fatal_error("$err error(s) encountered in parsing $path_ipxaddrconf\n");
	}
}

# ifconfig <if> alias <ipaddr>[/<masklen>] [<destaddr>] [-alias <ipaddr>]
$key = '^\s*ifconfig\s+';
$ifn = '[\w-]+';
$aliaskey = '^\s*ifconfig\s+';
$aliasn = 'alias';
$ipa = '\d+\.\d+\.\d+\.\d+';

sub parse_aliasaddrconf {
	local($linenum, $err, $c1, $c2, $c3, $c4);
	# Clear out any old state we might have
	undef %aliasifseen;         
	open(ADDRCONF, $path_addrconf) || &fatal_error("Can't open $path_addrconf\n");
	&syslog(LOG_NOTICE, "Parsing alias '$path_addrconf'\n") if $debug;
	$linenum = 0;
	while (<ADDRCONF>) {
		chop;
		$linenum++;
		if (/^#/ || /^s*$/) {
			next;
		}
		print "\t$linenum: $_ \n" if $debug;	
		 if (/^\s*ifconfig\s+([\.\/\w]+)\s*/) {
		     @flags = split(' ', $_); 
		     if($flags[2] != "alias" || $flags[2] != "alias") {
			    next;
		    }   
		    $c1 = $flags[1]; 
		    $c2 = $flags[2];
                    $c3 = $flags[3];		    
                    if($c2 != "alias" || $c2 != "-alias" ) {
			next;
		    }		       
		}
                else {
		    next; 
		}
                # Punt on duplicates -- shouldn't be any in /etc/if.conf 
		# but check anyway just to be sure. 
		if ($_ eq $aliasifconf{$c1}) {
                    print "\tduplicate: $linenum: $_ \n" if $debug;
		    next;
		}
		$aliasifconf{$1} = $_;		
		$ifalias{$1} = $c1;                
		# This assumes there is *NOT* 
		# both an alias and -alias command in /etc/if.conf 
		# for the same ipaddr on the same interface.
		if ( (!($aliasaddr{$c3})) && (!($aliasifcmd{$c2})) ) {
		    &syslog(LOG_NOTICE, "Configure alias: $_") if $debug;
                    print "Configure alias: $_\n" if $debug;
		    &configaliasif( $c1, $c2, $c3 );
		} 
		else {
                    print "\tNot calling configaliasif($c1, $c2, $c3)\n" if $debug;
		};
		$aliasifseen{$c1} = 1;
		$aliascmd{$c2} = 1;
		$aliasaddr{$c3} = 1; 
                 	
	}
	close(ADDRCONF);
	if ($err) {
	    &fatal_error("$err error(s) encountered in parsing $path_addrconf\n");
	}
}

sub usage {
	&fatal_error("Usage: pnpif [-f addrconf] [-p ipxaddrconf] [-i interval] [-d level] [-h]\n");
}

open (FD, "/bin/hostname |");
chop($hostname=<FD>);
close(FD);

$debug = 0;
$interval = 10;
$need_mcast_route = 1;
$mcast_if= "";
$change_ifconf = 0;
$path_addrconf = "/etc/if.conf";
$path_ipxaddrconf = "/etc/ipxif.conf";
$path_pidfile = "/var/run/pnpif.pid";
&openlog("pnpif.pl", LOG_PID | LOG_CONS, LOG_DAEMON);

while ($ARGV[0] =~ /^-/) {
	$_ = shift;
	if (/^-f(.*)/) {
		$path_addrconf = ($1 ? $1 : shift);
                printf "config file = $path_addrconf\n"; 
	}
	elsif (/^-p(.*)/) {
	        $path_ipxaddrconf = ($1 ? $1 : shift);
	}
	elsif (/^-i(.*)/) {
		$interval = ($1 ? $1 : shift);
	}
	elsif (/^-d(.*)/) {
		$debug = ($1 ? $1 : shift);
	}
	elsif (/^-h/) {
		&usage;
	}
	else {
		&usage;
	}
}

$SIG{'HUP'} = 'sighup_handler';
$SIG{'USR1'} = 'sigusr1_handler';

open(PIDFILE, "> $path_pidfile");
print PIDFILE "$$\n";
close(PIDFILE);

# Add the loopback first, for syslog
system("/sbin/ifconfig lo0 127.0.0.1/8 up");

# We need to make sure Ipsrd or mrouted is not running when we first start
# up or reconfigure

&stop_ipsrd;

undef %ifactive;

&handler(1);  #  Force a pass through to get the initial values

sleep 10;   #  Brief delay to allow local stuff to settle

$fw_firsttime = 1;

while (1) {
	if ($signal_pending) {
	    $signal_pending = 0;
	    &syslog(LOG_NOTICE, "Signal pending") if $debug;
	    &handler(0);
	}
	$SIG{'HUP'} = 'sighup_handler';        
	&scanifs;

	#
	# start ipsrd and mrouted 
	# if configuration have changed and we have added route 224.0.0.0
	#
	if ($change_ifconf && (!$need_mcast_route)) {
	    $change_ifconf = 0;
	    &start_mrouted;
	    &start_ipsrd;
	}
	if ($fw_firsttime == 1) {
	    &start_fw;
	    $fw_firsttime = 0;
	}
	sleep $interval;
}
